==> libro-codemirror-0.1.0.tgz: package/es/auto-complete/index.js <==
import { Prec } from '@codemirror/state';
import { keymap } from '@codemirror/view';
import { indentOrCompletion, indentOrTooltip } from "../indent.js";
import { completionConfig } from "./config.js";
import { completionState, State, setSelectedEffect } from "./state.js";
import { baseTheme } from "./theme.js";
import { completionPlugin, moveCompletionSelection, acceptCompletion, closeCompletion } from "./view.js";
export * from "./snippet.js";
export * from "./completion.js";
export * from "./view.js";
export * from "./word.js";
export * from "./closebrackets.js";

/// Returns an extension that enables autocompletion.
export function autocompletion() {
  var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return [completionState, completionConfig.of(config), completionPlugin, completionKeymapExt, baseTheme];
}

/// Basic keybindings for autocompletion.
///
///  - Ctrl-Space: [`startCompletion`](#autocomplete.startCompletion)
///  - Escape: [`closeCompletion`](#autocomplete.closeCompletion)
///  - ArrowDown: [`moveCompletionSelection`](#autocomplete.moveCompletionSelection)`(true)`
///  - ArrowUp: [`moveCompletionSelection`](#autocomplete.moveCompletionSelection)`(false)`
///  - PageDown: [`moveCompletionSelection`](#autocomplete.moveCompletionSelection)`(true, "page")`
///  - PageDown: [`moveCompletionSelection`](#autocomplete.moveCompletionSelection)`(true, "page")`
///  - Enter: [`acceptCompletion`](#autocomplete.acceptCompletion)
export var completionKeymap = [{
  key: 'Tab',
  run: indentOrCompletion,
  shift: indentOrTooltip
}, {
  key: 'Escape',
  run: closeCompletion
}, {
  key: 'ArrowDown',
  run: moveCompletionSelection(true)
}, {
  key: 'ArrowUp',
  run: moveCompletionSelection(false)
}, {
  key: 'PageDown',
  run: moveCompletionSelection(true, 'page')
}, {
  key: 'PageUp',
  run: moveCompletionSelection(false, 'page')
}, {
  key: 'Enter',
  run: acceptCompletion
}];
var completionKeymapExt = Prec.highest(keymap.computeN([completionConfig], function (state) {
  return state.facet(completionConfig).defaultKeymap ? [completionKeymap] : [];
}));

/// Get the current completion status. When completions are available,
/// this will return `"active"`. When completions are pending (in the
/// process of being queried), this returns `"pending"`. Otherwise, it
/// returns `null`.
export function completionStatus(state) {
  var cState = state.field(completionState, false);
  if (cState && cState.active.some(function (a) {
    return a.state === State.Pending;
  })) {
    return 'pending';
  }
  if (cState && cState.active.some(function (a) {
    return a.state !== State.Inactive;
  })) {
    return 'active';
  }
  return null;
}
var completionArrayCache = new WeakMap();

/// Returns the available completions as an array.
export function currentCompletions(state) {
  var _state$field;
  var open = (_state$field = state.field(completionState, false)) === null || _state$field === void 0 ? void 0 : _state$field.open;
  if (!open) {
    return [];
  }
  var completions = completionArrayCache.get(open.options);
  if (!completions) {
    completionArrayCache.set(open.options, completions = open.options.map(function (o) {
      return o.completion;
    }));
  }
  return completions;
}

/// Return the currently selected completion, if any.
export function selectedCompletion(state) {
  var _state$field2;
  var open = (_state$field2 = state.field(completionState, false)) === null || _state$field2 === void 0 ? void 0 : _state$field2.open;
  return open && open.selected >= 0 ? open.options[open.selected].completion : null;
}

/// Returns the currently selected position in the active completion
/// list, or null if no completions are active.
export function selectedCompletionIndex(state) {
  var _state$field3;
  var open = (_state$field3 = state.field(completionState, false)) === null || _state$field3 === void 0 ? void 0 : _state$field3.open;
  return open && open.selected >= 0 ? open.selected : null;
}

/// Create an effect that can be attached to a transaction to change
/// the currently selected completion.
export function setSelectedCompletion(index) {
  return setSelectedEffect.of(index);
}

==> libro-codemirror-0.2.2.tgz: package/es/auto-complete/index.js <==
/* eslint-disable @typescript-eslint/no-use-before-define */

import { Prec } from '@codemirror/state';
import { keymap } from '@codemirror/view';
import { indentOrCompletion, indentOrTooltip } from "../indent.js";
import { completionConfig } from "./config.js";
import { completionState, State, setSelectedEffect } from "./state.js";
import { baseTheme } from "./theme.js";
import { completionPlugin, moveCompletionSelection, acceptCompletion, closeCompletion } from "./view.js";
export * from "./snippet.js";
export * from "./completion.js";
export * from "./view.js";
export * from "./word.js";
export * from "./closebrackets.js";

/// Returns an extension that enables autocompletion.
export function autocompletion() {
  var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return [completionState, completionConfig.of(config), completionPlugin, completionKeymapExt, baseTheme];
}

/// Basic keybindings for autocompletion.
///
///  - Ctrl-Space: [`startCompletion`](#autocomplete.startCompletion)
///  - Escape: [`closeCompletion`](#autocomplete.closeCompletion)
///  - ArrowDown: [`moveCompletionSelection`](#autocomplete.moveCompletionSelection)`(true)`
///  - ArrowUp: [`moveCompletionSelection`](#autocomplete.moveCompletionSelection)`(false)`
///  - PageDown: [`moveCompletionSelection`](#autocomplete.moveCompletionSelection)`(true, "page")`
///  - PageDown: [`moveCompletionSelection`](#autocomplete.moveCompletionSelection)`(true, "page")`
///  - Enter: [`acceptCompletion`](#autocomplete.acceptCompletion)
export var completionKeymap = [{
  key: 'Tab',
  run: indentOrCompletion,
  shift: indentOrTooltip
}, {
  key: 'Escape',
  run: closeCompletion
}, {
  key: 'ArrowDown',
  run: moveCompletionSelection(true)
}, {
  key: 'ArrowUp',
  run: moveCompletionSelection(false)
}, {
  key: 'PageDown',
  run: moveCompletionSelection(true, 'page')
}, {
  key: 'PageUp',
  run: moveCompletionSelection(false, 'page')
}, {
  key: 'Enter',
  run: acceptCompletion
}];
var completionKeymapExt = Prec.highest(keymap.computeN([completionConfig], function (state) {
  return state.facet(completionConfig).defaultKeymap ? [completionKeymap] : [];
}));

/// Get the current completion status. When completions are available,
/// this will return `"active"`. When completions are pending (in the
/// process of being queried), this returns `"pending"`. Otherwise, it
/// returns `null`.
export function completionStatus(state) {
  var cState = state.field(completionState, false);
  return cState && cState.active.some(function (a) {
    return a.state === State.Pending;
  }) ? 'pending' : cState && cState.active.some(function (a) {
    return a.state !== State.Inactive;
  }) ? 'active' : null;
}
var completionArrayCache = new WeakMap();

/// Returns the available completions as an array.
export function currentCompletions(state) {
  var _state$field;
  var open = (_state$field = state.field(completionState, false)) === null || _state$field === void 0 ? void 0 : _state$field.open;
  if (!open) {
    return [];
  }
  var completions = completionArrayCache.get(open.options);
  if (!completions) {
    completionArrayCache.set(open.options, completions = open.options.map(function (o) {
      return o.completion;
    }));
  }
  return completions;
}

/// Return the currently selected completion, if any.
export function selectedCompletion(state) {
  var _state$field2;
  var open = (_state$field2 = state.field(completionState, false)) === null || _state$field2 === void 0 ? void 0 : _state$field2.open;
  return open && open.selected >= 0 ? open.options[open.selected].completion : null;
}

/// Returns the currently selected position in the active completion
/// list, or null if no completions are active.
export function selectedCompletionIndex(state) {
  var _state$field3;
  var open = (_state$field3 = state.field(completionState, false)) === null || _state$field3 === void 0 ? void 0 : _state$field3.open;
  return open && open.selected >= 0 ? open.selected : null;
}

/// Create an effect that can be attached to a transaction to change
/// the currently selected completion.
export function setSelectedCompletion(index) {
  return setSelectedEffect.of(index);
}
